// Beispiel Programm MFC_LogFile (C++ Window Projekt Microsoft Visual Studio 2010)
//
// Beschreibung: Verbindet mit einer LStep Express Steuerung auf COM1 und COM9 (siehe ConnectController)
//               Anschlieend kann ein relativer Move oder ein Fehler  (siehe SendError) ausgelst werden
//               Der fehlercode des Befehls wird in einer ListBox ausgegeben. Weiterhin ist es mglich,
//               das Protokollfenster sowie den Fehlerkanal (siehe EnableErrorChannel) zu aktiveren. Bei 
//               aktivem Fehlerkanal werden asynchrone Fehler in einer ListBox angezeigt. Soll eine 
//               Log-Datei geschrieben werden, kann die Anzahl der Zeilen und der Dateiname festgelegt 
//               werden (siehe OnBnClickedButtonCreateLog). Weiterhin kann die Funktionalitt der 
//               CallBackFunktionen gestestet werden. (siehe SetExtCallBackFct, SetOsziCallBackFct, 
//               OnMyOwnMessage, LSTEPExtCallBackFct, LSTEPCallBackFct)
//
// Erzeugt von: Andreas Kstler, 15.07.2014 , LANG GmbH & Co. KG

#include "stdafx.h"
#include "MFC_LogFile.h"
#include "MFC_LogFileDlg.h"
#include "afxdialogex.h"
#include "LStep4X.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

void CALLBACK LSTEPCallBackFct(char *pcData, int lMaxLen, int lChannelID);
int CALLBACK LSTEPExtCallBackFct(char *pcData, int lMaxLen, int lChannelID, void* pObject);

// Struktur zur bergabe von Fehlermeldungen
struct ErrorPars {
  TCHAR errText[255][255]; // Textpuffer fr Fehlerkanal
  void* pObject[255]; // Zwischenspeicher fr Fehlerobjekt
  int count;
}; 

ErrorPars ErrorList; // Fehler Liste
ControllerStruct Con[2]; // Globale Variablen fr LSTEPexpress Handling
static CRITICAL_SECTION cs;

HWND nWndID;

// CAboutDlg-Dialogfeld fr Anwendungsbefehl "Info"

class CAboutDlg : public CDialogEx
{
public:
  CAboutDlg();

  // Dialogfelddaten
  enum { IDD = IDD_ABOUTBOX };

protected:
  virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV-Untersttzung

// Implementierung
protected:
  DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialogEx(CAboutDlg::IDD)
{
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
  CDialogEx::DoDataExchange(pDX);
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialogEx)
END_MESSAGE_MAP()


// CC_MFC_LogFileDlg-Dialogfeld

CC_MFC_LogFileDlg::CC_MFC_LogFileDlg(CWnd* pParent /*=NULL*/)
  : CDialogEx(CC_MFC_LogFileDlg::IDD, pParent)
  , LineCount(0)
{
  m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CC_MFC_LogFileDlg::DoDataExchange(CDataExchange* pDX)
{
  CDialogEx::DoDataExchange(pDX);
  DDX_Text(pDX, IDC_EDIT_LINE_COUNT, LineCount);
  DDV_MinMaxUInt(pDX, LineCount, 0, 1000000);
  DDX_Control(pDX, IDC_EDIT_FILE_NAME, FileName);
  DDX_Control(pDX, IDC_LIST_ERROR_NUMBERS, ListBoxErrorNumbers);
  DDX_Control(pDX, IDC_LIST_ERROR_CHANNEL_MESSAGES, ListBoxErrorChannelMsg);
  DDX_Control(pDX, IDC_EDIT_LINE_COUNT, Edit_LineCount);
  DDX_Control(pDX, IDC_CHECK_ERROR_CHANNEL, CheckBoxErrorChannel);
  DDX_Control(pDX, IDC_CHECK_PROT_WINDOW, CheckBoxProtocolWindow);
}

#define WM_MYOWNMESSAGE (WM_USER + 100)

BEGIN_MESSAGE_MAP(CC_MFC_LogFileDlg, CDialogEx)
  ON_WM_SYSCOMMAND()
  ON_WM_PAINT()
  ON_WM_QUERYDRAGICON()
  ON_MESSAGE(WM_MYOWNMESSAGE, OnMyOwnMessage)
  ON_BN_CLICKED(IDC_BUTTON_CONNECT, &CC_MFC_LogFileDlg::OnBnClickedButtonConnect)
  ON_BN_CLICKED(IDC_BUTTON_CONNECT2, &CC_MFC_LogFileDlg::OnBnClickedButtonConnect2)
  ON_BN_CLICKED(IDC_BUTTON_DISCONNECT, &CC_MFC_LogFileDlg::OnBnClickedButtonDisconnect)
  ON_BN_CLICKED(IDC_BUTTON_DISCONNECT2, &CC_MFC_LogFileDlg::OnBnClickedButtonDisconnect2)
  ON_BN_CLICKED(IDC_BUTTON_MOVE_REL, &CC_MFC_LogFileDlg::OnBnClickedButtonMoveRel)
  ON_BN_CLICKED(IDC_BUTTON_SEND_ERROR, &CC_MFC_LogFileDlg::OnBnClickedButtonSendError)
  ON_BN_CLICKED(IDC_BUTTON_SEND_ERROR2, &CC_MFC_LogFileDlg::OnBnClickedButtonSendError2)
  ON_BN_CLICKED(IDC_BUTTON_CREATE_LOG, &CC_MFC_LogFileDlg::OnBnClickedButtonCreateLog)
  ON_BN_CLICKED(IDC_BUTTON_FREE_LOG, &CC_MFC_LogFileDlg::OnBnClickedButtonFreeLog)
  ON_BN_CLICKED(IDC_BUTTON_CLEAR_MSG, &CC_MFC_LogFileDlg::OnBnClickedButtonClearMsg)
  ON_BN_CLICKED(IDC_CHECK_ERROR_CHANNEL, &CC_MFC_LogFileDlg::OnBnClickedCheckErrorChannel)
  ON_BN_CLICKED(IDC_CHECK_PROT_WINDOW, &CC_MFC_LogFileDlg::OnBnClickedCheckProtWindow)
  ON_WM_DESTROY()
END_MESSAGE_MAP()


// CC_MFC_LogFileDlg-Meldungshandler

BOOL CC_MFC_LogFileDlg::OnInitDialog()
{
  CDialogEx::OnInitDialog();

  // Hinzufgen des Menbefehls "Info..." zum Systemmen.

  // IDM_ABOUTBOX muss sich im Bereich der Systembefehle befinden.
  ASSERT((IDM_ABOUTBOX & 0xFFF0) == IDM_ABOUTBOX);
  ASSERT(IDM_ABOUTBOX < 0xF000);

  CMenu* pSysMenu = GetSystemMenu(FALSE);
  if (pSysMenu != NULL)
  {
    BOOL bNameValid;
    CString strAboutMenu;
    bNameValid = strAboutMenu.LoadString(IDS_ABOUTBOX);
    ASSERT(bNameValid);
    if (!strAboutMenu.IsEmpty())
    {
      pSysMenu->AppendMenu(MF_SEPARATOR);
      pSysMenu->AppendMenu(MF_STRING, IDM_ABOUTBOX, strAboutMenu);
    }
  }

  // Symbol fr dieses Dialogfeld festlegen. Wird automatisch erledigt
  //  wenn das Hauptfenster der Anwendung kein Dialogfeld ist
  SetIcon(m_hIcon, TRUE);			// Groes Symbol verwenden
  SetIcon(m_hIcon, FALSE);		// Kleines Symbol verwenden

  // Initialisierung der LSTEP Handling Variable
  InitializeCriticalSection(&cs);
  Con[0].Strg = NULL;
  Con[0].Strg = new CLStep4X();
  Con[0].Connected = false;

  Con[1].Strg = NULL;
  Con[1].Strg = new CLStep4X();
  Con[1].Connected = false;

  ErrorList.count = 0;

  nWndID = m_hWnd;

  FileName.SetWindowTextW(_T("LStep4.log"));
  CheckBoxErrorChannel.SetCheck(BST_CHECKED);
  CheckBoxProtocolWindow.SetCheck(BST_CHECKED);
  return TRUE;  // TRUE zurckgeben, wenn der Fokus nicht auf ein Steuerelement gesetzt wird
}

void CC_MFC_LogFileDlg::OnSysCommand(UINT nID, LPARAM lParam)
{
  if ((nID & 0xFFF0) == IDM_ABOUTBOX)
  {
    CAboutDlg dlgAbout;
    dlgAbout.DoModal();
  }
  else
  {
    CDialogEx::OnSysCommand(nID, lParam);
  }
}

// Wenn Sie dem Dialogfeld eine Schaltflche "Minimieren" hinzufgen, bentigen Sie
//  den nachstehenden Code, um das Symbol zu zeichnen. Fr MFC-Anwendungen, die das 
//  Dokument/Ansicht-Modell verwenden, wird dies automatisch ausgefhrt.

void CC_MFC_LogFileDlg::OnPaint()
{
  if (IsIconic())
  {
    CPaintDC dc(this); // Gertekontext zum Zeichnen

    SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

    // Symbol in Clientrechteck zentrieren
    int cxIcon = GetSystemMetrics(SM_CXICON);
    int cyIcon = GetSystemMetrics(SM_CYICON);
    CRect rect;
    GetClientRect(&rect);
    int x = (rect.Width() - cxIcon + 1) / 2;
    int y = (rect.Height() - cyIcon + 1) / 2;

    // Symbol zeichnen
    dc.DrawIcon(x, y, m_hIcon);
  }
  else
  {
    CDialogEx::OnPaint();
  }
}

// Die System ruft diese Funktion auf, um den Cursor abzufragen, der angezeigt wird, whrend der Benutzer
//  das minimierte Fenster mit der Maus zieht.
HCURSOR CC_MFC_LogFileDlg::OnQueryDragIcon()
{
  return static_cast<HCURSOR>(m_hIcon);
}

// Standard Callbackfunktion z.B. fr ErrorChannel Massages
// Die Callbackfunktion sollte nur zur Signalisierung bzw. zum weiterreichen von Fehlermeldungen verwendet werden!
// Aus dieser Callbackfunktion kann kein Rckschluss auf die Steuerung geschlossen werden, von der der Callback
// ausgelst wurde.
//
// Wurde einer Steuerung eine Standard Callbackfunktion sowie eine Extended Callbackfunktion zugewiesen, wird nur
// die Extended Callbackfunktion ausgefhrt.
void CALLBACK LSTEPCallBackFct(char *pcData, int lMaxLen, int lChannelID)
{    
  //  Bedeutung der ChannelID der Callbackfunktion
  //  1 = Oszilloskop Daten
  //  2 = Fehler/Zusatzhinweise der Steuerung
  //  3 = Oszilloskop Aufzeichnungsinformationen
  //  4... = Noch nicht belegt   
  if (lChannelID == 2)
  { 
    while (TryEnterCriticalSection(&cs) == false);
    wcscpy_s(ErrorList.errText[ErrorList.count], CString(pcData, lMaxLen)); // Kopiere Fehlerstring in Puffer
    ErrorList.pObject[ErrorList.count] = NULL; // pObject steht in dieser Callbackfunktion nicht zur Verfgung
    ErrorList.count++;
    LeaveCriticalSection(&cs);				
    PostMessageW(nWndID, WM_MYOWNMESSAGE, WPARAM(&ErrorList), 0); // Lse Message fr Schreiben in Fenster aus
  }
}

// Extended Callbackfunktion z.B. fr ErrorChannel Massages
// Die Callbackfunktion sollte nur zur Signalisierung bzw. zum weiterreichen von Fehlermeldungen verwendet werden!
// ber das Objekt hinter void* pObject kann die auslsende Steuerung identifiziert werden. Der Zeiger auf das Objekt
// wird in der Funktion SetExtCallBackFct() als Parameter bergeben.
//
// Wurde einer Steuerung eine Standard Callbackfunktion sowie eine Extended Callbackfunktion zugewiesen, wird nur
// die Extended Callbackfunktion ausgefhrt.
int CALLBACK LSTEPExtCallBackFct(char *pcData, int lMaxLen, int lChannelID, void* pObject)
{
  //  Bedeutung der ChannelID der Callbackfunktion
  //  1 = Oszilloskop Daten
  //  2 = Fehler/Zusatzhinweise der Steuerung
  //  3 = Oszilloskop Aufzeichnungsinformationen
  //  4... = Noch nicht belegt   
  if (lChannelID == 2)
  {
    while (TryEnterCriticalSection(&cs) == false);
    wcscpy_s(ErrorList.errText[ErrorList.count], CString(pcData, lMaxLen)); // Kopiere Fehlerstring in Puffer
    ErrorList.pObject[ErrorList.count] = pObject; // Kopiere Zeiger aus pObject
    ErrorList.count++;
    LeaveCriticalSection(&cs);				
    PostMessageW(nWndID, WM_MYOWNMESSAGE, WPARAM(&ErrorList), 0); // Lse Message fr Schreiben in Fenster aus
  }
  return 0;
}


// Funktion zum Verarbeiten der Callback-Funktionsaufrufe
LRESULT CC_MFC_LogFileDlg::OnMyOwnMessage(WPARAM wParam, LPARAM lParam)
{   
  TCHAR strTranslated[255];
  void* pObject[255];
  CString AchsNr;
  int TokenPos = 0;
  int MessageType, Achse = 0;
  CString DataStringOut;
  CLStep4X *LS;
  CListBox *ListBoxErrorChannelMsg = (CListBox*) GetDlgItem(IDC_LIST_ERROR_CHANNEL_MESSAGES);
  int i;

  ErrorPars *EList;
  EList = (ErrorPars*)wParam;
  LS = Con[0].Strg; // Initialisiere LS mit Con[0] Steuerung falls Standard Callback Funktion verwendet wird 

  while (TryEnterCriticalSection(&cs) == false); 	

  CStringArray StrArrTemp;
  // kopiere Elemente aus EList und setze Zhler zurck
  for(i=0;i < EList->count;i++) 
  {
    StrArrTemp.Add(EList->errText[i]);
    pObject[i] = EList->pObject[i];
  }

  EList->count = 0;
  LeaveCriticalSection(&cs);	

  // Werte Nachrichten aus
  for(i=0;i<StrArrTemp.GetCount();i++)
  {
    // Wurde ein Objekt zur Nachricht bergeben, dann Verarbeite anhand des bergebenen Objekts
    if(pObject[i] != NULL) 
    {
      LS = (CLStep4X*) pObject[i];
      // Hole Control Nr und schreibe in ListBox
      _itot_s(LS->GetLSID(),strTranslated,10);
      DataStringOut = strTranslated;
      ListBoxErrorChannelMsg->InsertString(ListBoxErrorChannelMsg->GetCount( ), _T("Control Nr: ") + DataStringOut);
    }

    LS->TranslateErrMsg(((CString)StrArrTemp.GetAt(i)).GetBuffer(0), DataStringOut.GetBuffer(255), 255); // bersetze Fehler in eine lesbare Meldung
    ListBoxErrorChannelMsg->InsertString(ListBoxErrorChannelMsg->GetCount( ), DataStringOut); // Fge Meldung in listBox ein

    // Falls die API-Funktion nicht zur Auswertung verwendet werden soll:
    TokenPos = 0;
    MessageType = _tstoi(StrArrTemp[i].Tokenize(_T("\t"),TokenPos));  // Konvertiere 1. Teilstring in Nachrichtentyp
    //Aufschlsselung der Nachrichtentypen
    //  0 = in der Regel einem Fehler
    //  99 = Hinweis zu einem Fehlern
    switch (MessageType) // Fhre Nachrichtenabhngigen Code aus
    {
    case 0: // Fehler

      // Aufschlsselung des StrArrTemp[i] Element, dessen Eintrge durch \t getrennt sind
      //  1.	Die Achsnummer. 0 entspricht der X-Achse  5 entspricht einer nicht Achsgebundenen Meldung
      //  2.	Fehlernummer. Die Fehlerliste kann in der Dokumentation nachgelesen werden.
      //  3.	Zeit in Sekunden seit dem Start der Steuerung.
      //  4.	Zeit in Millisekunden als Bruchteil der Betriebssekunden.

      Achse = _tstoi(StrArrTemp[i].Tokenize(_T("\t"),TokenPos)); // Wandle Achsnummer in Zahl
      if (Achse > 4) // ist Nummer grer 4 ist es keine achsspezifische Fehlernummer
      {
        AchsNr = _T("-");
      }
      else
      {
        AchsNr.Format(_T("%d"), Achse);
      }

      // Codiere eigene Fehlermeldung
      DataStringOut = _T(  "Achse ") + AchsNr;
      DataStringOut = DataStringOut + _T("; Fehler Nr ") + StrArrTemp[i].Tokenize(_T("\t"),TokenPos);
      DataStringOut = DataStringOut + _T("; Betriebssekunden ") + StrArrTemp[i].Tokenize(_T("\t"),TokenPos);
      DataStringOut = DataStringOut + _T("s ") + StrArrTemp[i].Tokenize(_T("\t"),TokenPos) + _T("ms");

      ListBoxErrorChannelMsg->InsertString(ListBoxErrorChannelMsg->GetCount( ), DataStringOut); // Fge Meldung in listBox ein
      break;
    case 99: // Hinweis

      // Aufschlsselung der StrArrTemp[i] elemente, die durch \t getrennt sind
      // 1. String als Klartext

      DataStringOut = _T("Hinweis: ") + StrArrTemp[i].Tokenize(_T("\t"),TokenPos);
      ListBoxErrorChannelMsg->InsertString(ListBoxErrorChannelMsg->GetCount( ), DataStringOut); // Fge Meldung in listBox ein
      break;
    default:
      break;
    }
  }
  return 0;

}

void CC_MFC_LogFileDlg::OnBnClickedButtonConnect()
{
  ConnectController(_T("com1"), _T("DEU"), &Con[0], true);
}

void CC_MFC_LogFileDlg::OnBnClickedButtonConnect2()
{
  ConnectController(_T("com9"), _T("ENG"), &Con[1], true);
}

void CC_MFC_LogFileDlg::ConnectController(TCHAR *pCOM_Name, TCHAR *pLanguage, ControllerStruct* pController, bool ExtCallBack)
{
  int nRe = 0;
  if(ExtCallBack)
  {
    // Setze Extendet CallbackFunktion
    pController->Strg->SetExtCallBackFct((void*)LSTEPExtCallBackFct, (void*) pController->Strg);
  }
  else
  {
    // Setze Standard CallbackFunktion
    pController->Strg->SetOsziCallBackFct((void*)LSTEPCallBackFct);
  }

  pController->Strg->SetLanguageW(pLanguage);  // Setze Sprache
  nRe = pController->Strg->ConnectSimpleW(5, pCOM_Name, 115200, (CheckBoxProtocolWindow.GetCheck()==BST_CHECKED));  // Baue eine Verbindung zur Steuerung auf
  if (nRe==0) pController->Connected = true;
  if(pController->Connected)
  {
    EnableErrorChannel(pController);
  }
}

void CC_MFC_LogFileDlg::OnBnClickedButtonDisconnect()
{
  // Trenne Verbindung
  Con[0].Connected = false;
  Con[0].Strg->Disconnect();
}

void CC_MFC_LogFileDlg::OnBnClickedButtonDisconnect2()
{
  // Trenne Verbindung
  Con[1].Connected = false;
  Con[1].Strg->Disconnect();
}

void CC_MFC_LogFileDlg::OnBnClickedButtonMoveRel()
{
  CString csErrorNumber;
  int ErrorNumber = 0;
  Con[0].Strg->MoveRel(1.0, 1.0, 1.0, 1.0, true); // Fhre relativen Move aus
  Con[0].Strg->GetError(&ErrorNumber);  // Rufe Fehler ab
  csErrorNumber.Format(_T("%d"),ErrorNumber);
  ListBoxErrorNumbers.InsertString(ListBoxErrorNumbers.GetCount( ), csErrorNumber); // Fge Fehler in listBox ein
}

void CC_MFC_LogFileDlg::OnBnClickedButtonSendError()
{
  SendError(&Con[0]);
}

void CC_MFC_LogFileDlg::OnBnClickedButtonSendError2()
{
  SendError(&Con[1]);
}

void CC_MFC_LogFileDlg::SendError(ControllerStruct* pController)
{
  CString csErrorNumber;
  int ErrorNumber = 0;
  pController->Strg->SendStringW(_T("!FEHLER\r"), NULL, 0, false, 100); // Da die Steuerung diesen Befehl nicht kennt, wir ein Fehler erzeugt
  pController->Strg->GetError(&ErrorNumber);  // Rufe Fehlercode ab
  csErrorNumber.Format(_T("%d"),ErrorNumber);
  ListBoxErrorNumbers.InsertString(ListBoxErrorNumbers.GetCount( ), csErrorNumber);
}
void CC_MFC_LogFileDlg::OnBnClickedButtonCreateLog()
{
  CString csLineCount;
  CString csFileName;
  TCHAR *cFileName;

  Edit_LineCount.GetWindowTextW(csLineCount);
  Con[0].Strg->SetExtValue(4, _wtoi(csLineCount)); // Setze Zeilenlnge, ab deren berschreitung eine neue Textdatei erzeugt wird

  Con[0].Strg->SetWriteLogTextFNW(false, NULL); // Deaktiviere Log-File

  FileName.GetWindowTextW(csFileName);
  cFileName = csFileName.GetBuffer();
  Con[0].Strg->SetWriteLogTextFNW(true, cFileName); // Aktiviere Log-File
  csFileName.ReleaseBuffer();
}

void CC_MFC_LogFileDlg::OnBnClickedButtonFreeLog()
{
  Con[0].Strg->SetWriteLogTextFNW(false, NULL); // Deaktivere Log-File
}

void CC_MFC_LogFileDlg::OnBnClickedButtonClearMsg()
{
  ListBoxErrorNumbers.ResetContent();
  ListBoxErrorChannelMsg.ResetContent();
}

void CC_MFC_LogFileDlg::OnBnClickedCheckErrorChannel()
{
  EnableErrorChannel(&Con[0]);
  EnableErrorChannel(&Con[1]);
}

void CC_MFC_LogFileDlg::EnableErrorChannel(ControllerStruct* pController)
{
  if(pController->Connected)
  {
    if (CheckBoxErrorChannel.GetCheck() == BST_CHECKED)
    {
      pController->Strg->SendStringW(_T("!errorchannel 2\r"), NULL, 0, false, 100); // Aktiviere ErrorChannel
    }
    else
    {
      pController->Strg->SendStringW(_T("!errorchannel 0\r"), NULL, 0, false, 100); // Deaktiviere ErrorChannel
    }
  }
}

void CC_MFC_LogFileDlg::OnBnClickedCheckProtWindow()
{
  Con[0].Strg->SetShowProt(CheckBoxProtocolWindow.GetCheck() == BST_CHECKED);
  Con[1].Strg->SetShowProt(CheckBoxProtocolWindow.GetCheck() == BST_CHECKED);
}

void CC_MFC_LogFileDlg::OnDestroy()
{
  CDialogEx::OnDestroy();

  if(Con[0].Strg)
  {
    if (Con[0].Connected) Con[0].Strg->Disconnect();
    delete Con[0].Strg;
  }

  if(Con[1].Strg)
  {
    if (Con[1].Connected) Con[1].Strg->Disconnect();
    delete Con[1].Strg;
  }
  DeleteCriticalSection(&cs);
}
